#include "nes.hh"

void bisqnes::load(const char *fn)
{
    FILE *fp = fopen(fn, "rb");
    if(!fp)return;

    if(fgetc(fp)=='N'
    && fgetc(fp)=='E'
    && fgetc(fp)=='S'
    && fgetc(fp)=='\32')
    {
        rom16count = fgetc(fp);
        vrom8count = fgetc(fp);
        ctrlbyte = fgetc(fp);
        mappernum = fgetc(fp) | (ctrlbyte>>4);
        fseek(fp, 0x10, SEEK_SET);
        
        if(mappernum >= 0x40)
            mappernum &= 15;
        
        fprintf(stderr, "mappernum %u, 8krom=%u, 4kvrom=%u\n", mappernum, rom16count*2, vrom8count*2);
        map<unsigned, mapper *>::iterator i;
        i = mappers.find(mappernum);
        if(i == mappers.end())
        {
            fprintf(stderr, "(mapper unknown)\n");
            i = mappers.find(0);
        }
        
        mapp = i->second;
        mapp->vrom4kcount = vrom8count * 2;
        mapp->rom8kcount = rom16count * 2;
        
        if(rom)delete[] rom;
        rom = new BYTE[rom16count * 16384];
        fread(&rom[0], 16384, rom16count, fp);
        
        ppu.resizevrom(vrom8count * 8192);
        fread(ppu.vromptr(), 8192, vrom8count, fp);
    }
    
    fclose(fp);
    
    // ROM pages ok, cpu reset etc.
    poweron();
}

void bisqnes::sta_load(const char *fn)
{
    FILE *fp = fopen(fn, "rb");
    if(!fp)return;
    fread(&sram, 1, 0x2000, fp);
    fread(&ram, 1, 0x800, fp);
    cpu.PC.B.h = fgetc(fp);
    cpu.PC.B.l = fgetc(fp);
    cpu.A = fgetc(fp);
    cpu.P = fgetc(fp);
    cpu.X = fgetc(fp);
    cpu.Y = fgetc(fp);
    cpu.S = fgetc(fp);
    fread(&ppu.spr_ram,1, 0x100, fp);
    fread(&ppu.vram,   1, 0x2000, fp);
    fread(&ppu.namtab, 4, 0x400, fp);    
    fread(&ppu.palette,2, 0x10, fp);
    
    long pos = ftell(fp);
    
    /*    
    mapp->reset();
    mapp->initialization();
    
    fseek(fp, 0x58, SEEK_CUR);
    for(unsigned a=0; a<4; ++a)
        mapp->maprom(1, a, fgetc(fp));
    
    fseek(fp, 0x10, SEEK_CUR);
    for(unsigned a=0; a<4; ++a)
        mapp->maprom(1, a+4, fgetc(fp));
    
    bool tmp = mapp->debug;
    mapp->debug=true;
    mapp->update();
    mapp->debug=tmp;
    */
    
    // debugging
    
// ST7:  00 00 00 00  02 03 0E 0F   05 08 09 0F  16 17 22 23 
// ST8:  00 00 00 00  02 03 0E 0F   01 08 09 0F  16 17 1A 1B

    fseek(fp, 0, SEEK_END);
    fprintf(stderr, "%ld unknown bytes at sta file:\n", ftell(fp)-pos);
    fseek(fp, pos, SEEK_SET);
    for(int c=0,b=0;;++b)
    {
        int a = fgetc(fp);
        if(a == EOF)break;
        if(b==8)fprintf(stderr," ");else if(b==16){b=0;fprintf(stderr,"\n");if(++c==4){c=0;fprintf(stderr,"\n");}}
        fprintf(stderr, "%02X ", a);
    }
    
    Wr(0x4015, 0x1F);
    
    fclose(fp);
    fprintf(stderr, "\nLoaded %s.\n", fn);
}

void bisqnes::run()
{
    fBYTE divider = Clock->CPUclockDivider;
    BYTE keycounter = 0;
    for(;;)
    {
        // PPU ticks faster!
        if(ppu.tick(divider))
            cpu.Int(INT_NMI);
        
        if(papu.tick())
        	cpu.Int(INT_IRQ);
        
        cpu.Tick();
        if((!++keycounter) && pads.quit())break;
    }
}
